package com.qmth.sop.server.api;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.sop.business.activiti.DefaultInstanceConvertToMultiInstance;
import com.qmth.sop.business.bean.FormPropertyBean;
import com.qmth.sop.business.bean.result.FlowFormWidgetResult;
import com.qmth.sop.business.bean.result.FlowResult;
import com.qmth.sop.business.bean.result.FlowTaskResult;
import com.qmth.sop.business.entity.TFCustomFlow;
import com.qmth.sop.business.entity.TFCustomFlowEntity;
import com.qmth.sop.business.entity.TFFlowApprove;
import com.qmth.sop.business.entity.TFFlowLog;
import com.qmth.sop.business.service.*;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.*;
import com.qmth.sop.common.util.GsonUtil;
import com.qmth.sop.common.util.JacksonUtil;
import io.swagger.annotations.*;
import org.activiti.bpmn.model.BpmnModel;
import org.activiti.bpmn.model.FlowNode;
import org.activiti.bpmn.model.UserTask;
import org.activiti.engine.FormService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.impl.bpmn.behavior.MultiInstanceActivityBehavior;
import org.activiti.engine.repository.DeploymentBuilder;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipInputStream;

@Api(tags = "测试流程表单htmlController")
@RestController
@RequestMapping("/activiti/form/html")
public class ActivitiFromHtmlController {
    private final static Logger log = LoggerFactory.getLogger(ActivitiFromHtmlController.class);

    @Resource
    ActivitiConsumerService activitiConsumerService;

    @Resource
    RepositoryService repositoryService;

    @Resource
    FormService formService;

    @Resource
    TaskService taskService;

    @Resource
    TFCustomFlowService tfCustomFlowService;

    @Resource
    RuntimeService runtimeService;

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    @Resource
    TFFlowApproveService tfFlowApproveService;

    @Resource
    TFFlowLogService tfFlowLogService;

    /**
     * 注册流程
     *
     * @return
     */
    @ApiOperation(value = "创建流程接口")
    @RequestMapping(value = "/createDeployment", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> createDeployment() throws IOException {
        DeploymentBuilder builder = repositoryService.createDeployment();
        ClassPathResource resource = new ClassPathResource("testform1.zip");
        ZipInputStream zip = new ZipInputStream(resource.getInputStream());
        builder.addZipInputStream(zip);
        return flowStart(builder.deploy().getId(), null, null);
    }

    @ApiOperation(value = "获取流程部署信息")
    @RequestMapping(value = "/flow/deployment/data", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getFlowData() {
        return Collections.singletonMap("data", tfCustomFlowService.findFlowDeploymentList());
    }

    @ApiOperation(value = "获取流程表单属性接口")
    @RequestMapping(value = "/flow/formPropertie/get", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> flowStart(@ApiParam(value = "流程部署id") @RequestParam(required = false) String flowDeploymentId,
                                         @ApiParam(value = "流程id") @RequestParam(required = false) Long flowId,
                                         @ApiParam(value = "流程任务id") @RequestParam(required = false) Long taskId) {
        Map<String, Object> map = new HashMap<>();
        FlowResult flowResult = null;
        TFCustomFlow tfCustomFlow = null;
        if (Objects.nonNull(flowDeploymentId)) {
            String processDefinitionId = null;
            tfCustomFlow = tfCustomFlowService.getOne(new QueryWrapper<TFCustomFlow>().lambda().eq(TFCustomFlow::getFlowDeploymentId, flowDeploymentId));
            Optional.ofNullable(tfCustomFlow.getFlowProcessVar()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有部署数据！"));
            if (Objects.isNull(tfCustomFlow.getFlowDefinitionId())) {
                processDefinitionId = activitiConsumerService.findProcessDefinitionIdByDeploymentId(flowDeploymentId);
                tfCustomFlow.setFlowDefinitionId(processDefinitionId);
                tfCustomFlowService.updateById(tfCustomFlow);
            } else {
                processDefinitionId = tfCustomFlow.getFlowDefinitionId();
            }
            flowResult = JSONObject.parseObject(tfCustomFlow.getFlowProcessVar(), FlowResult.class);
            map = this.getFlowFormPropertie(flowResult, map, 1);
        } else {
            List<Task> taskList = null;
            if (Objects.nonNull(flowId) && !Objects.equals(flowId, "")) {
                taskList = taskService.createTaskQuery().processInstanceId(String.valueOf(flowId)).list();
            } else if (Objects.nonNull(taskId) && !Objects.equals(taskId, "")) {
                taskList = taskService.createTaskQuery().taskId(String.valueOf(taskId)).list();
            }
            if (!CollectionUtils.isEmpty(taskList)) {
                if (Objects.isNull(flowId)) {
                    flowId = Long.parseLong(taskList.get(0).getProcessInstanceId());
                }
                TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, flowId));
                Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有实例数据！"));
                tfCustomFlow = tfCustomFlowService.getById(tfCustomFlowEntity.gettFCustomFlowId());
                Optional.ofNullable(tfCustomFlow.getFlowProcessVar()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有部署数据！"));

                flowResult = JSONObject.parseObject(tfCustomFlowEntity.getFlowProcessVar(), FlowResult.class);
                LinkedHashMap<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
                for (Task task : taskList) {
                    FlowTaskResult currFlowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(setupMap.get(task.getTaskDefinitionKey())), FlowTaskResult.class);
                    map = this.getFlowFormPropertie(flowResult, map, currFlowTaskResult.getSetup());
                    break;
                }
            }
        }
        map.put(SystemConstant.FLOW_DEPLOYMENT_ID, tfCustomFlow.getFlowDeploymentId());
        map.put(SystemConstant.ID, tfCustomFlow.getId());
        return map;
    }

    /**
     * 获取表单属性
     *
     * @param flowResult
     * @param map
     * @param setup
     * @return
     */
    public Map<String, Object> getFlowFormPropertie(FlowResult flowResult, Map<String, Object> map, int setup) {
        Map<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResultTemp = GsonUtil.fromJson(GsonUtil.toJson(entry.getValue()), FlowTaskResult.class);
            if (flowTaskResultTemp.getSetup().intValue() == setup) {
                map.put(SystemConstant.FORM_PROPERTIES, flowTaskResultTemp);
                break;
            }
        }
        return map;
    }

    @ApiOperation(value = "启动流程接口")
    @RequestMapping(value = "/start/flow", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> startFlow(@ApiParam(value = "流程部署id", required = true) @RequestParam String flowDeploymentId,
                                         @ApiParam(value = "流程操作", required = true) @RequestParam FlowApprovePassEnum approve,
                                         @ApiParam(value = "流程id") @RequestParam(required = false) Long flowId,
                                         @ApiParam(value = "流程表单数据") @RequestParam(required = false) String formProperties,
                                         @ApiParam(value = "流程审批人") @RequestParam(required = false) List<Long> approveUserIds,
                                         @ApiParam(value = "上传文件") @RequestParam(required = false) MultipartFile file,
                                         HttpServletRequest request) {
        Map<String, Object> map = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        map.putAll(formDataMap);

        List<String> approveIdList = approveUserIds.stream().map(s -> String.valueOf(s)).collect(Collectors.toList());
        TFCustomFlow tfCustomFlow = tfCustomFlowService.getOne(new QueryWrapper<TFCustomFlow>().lambda().eq(TFCustomFlow::getFlowDeploymentId, flowDeploymentId));
        Optional.ofNullable(tfCustomFlow.getFlowProcessVar()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有部署数据！"));
        String processFlowId = null;
        TFCustomFlowEntity tfCustomFlowEntity = null;
        TFFlowApprove tfFlowApprove = null;
        if (Objects.isNull(flowId)) {
            map.put(DefaultInstanceConvertToMultiInstance.DEFAULT_ASSIGNEE_LIST + 1, approveIdList);
            ProcessInstance processInstance = runtimeService.startProcessInstanceById(tfCustomFlow.getFlowDefinitionId(), map);
            processFlowId = processInstance.getId();
            tfCustomFlowEntity = new TFCustomFlowEntity(tfCustomFlow.getId(), null);
            tfFlowApprove = new TFFlowApprove(null, Long.parseLong(approveIdList.get(approveIdList.size() - 1)), FlowStatusEnum.valueOf(approve.name()), null);
        } else {
            processFlowId = String.valueOf(flowId);
            tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, flowId));
            tfFlowApprove = tfFlowApproveService.getOne(new QueryWrapper<TFFlowApprove>().lambda().eq(TFFlowApprove::getFlowId, flowId));
        }
        Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有实例数据！"));
        Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有审批数据！"));

        List<TFFlowLog> tfFlowLogList = new ArrayList<>();
        FlowResult flowResult = null;
        if (Objects.nonNull(tfCustomFlowEntity.getFlowProcessVar())) {
            flowResult = JSONObject.parseObject(tfCustomFlowEntity.getFlowProcessVar(), FlowResult.class);
        } else {
            flowResult = JSONObject.parseObject(tfCustomFlow.getFlowProcessVar(), FlowResult.class);
        }
        LinkedHashMap<String, FlowTaskResult> setupMap = flowResult.getSetupMap();

        tfCustomFlowEntity.setFlowId(Long.parseLong(processFlowId));
        tfFlowApprove.setFlowId(Long.parseLong(processFlowId));
        tfFlowApprove.setApproveMaxSetup(setupMap.size());
        List<Task> taskList = taskService.createTaskQuery().processInstanceId(processFlowId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            for (Task task : taskList) {
                String currActivityId = task.getTaskDefinitionKey();
                BpmnModel bpmnModel = repositoryService.getBpmnModel(task.getProcessDefinitionId());
                FlowNode currFlow = (FlowNode) bpmnModel.getMainProcess().getFlowElement(currActivityId);
                FlowTaskResult currFlowTaskResult = null, nextFlowTaskResult = null;
                Integer nrOfCompletedInstances = null, nrOfInstances = null;
                boolean multiInstance = false;
                if (currFlow instanceof UserTask) {
                    if (currFlow.getBehavior() instanceof MultiInstanceActivityBehavior) {
                        nrOfCompletedInstances = (Integer) taskService.getVariable(task.getId(), DefaultInstanceConvertToMultiInstance.NR_OF_COMPLETED_INSTANCES);
                        nrOfCompletedInstances++;
                        nrOfInstances = (Integer) taskService.getVariable(task.getId(), DefaultInstanceConvertToMultiInstance.NR_OF_INSTANCES);
                        multiInstance = true;
                    }
                    if (Objects.isNull(formProperties)) {
                        currFlowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(setupMap.get(task.getTaskDefinitionKey())), FlowTaskResult.class);
                    } else {
                        currFlowTaskResult = JSONObject.parseObject(formProperties, FlowTaskResult.class);
                    }
                    int currSetup = currFlowTaskResult.getSetup().intValue();
                    currSetup = currSetup == setupMap.size() - 1 ? 0 : currSetup + 1;

                    //流程流水日志
                    TFFlowLog tfFlowLog = new TFFlowLog(null, Long.parseLong(task.getAssignee()), null,
                            FlowApproveOperationEnum.valueOf(approve.name()), currFlowTaskResult.getSetup(),
                            FlowApproveSetupEnum.START.getTitle(), Long.parseLong(processFlowId),
                            Long.parseLong(task.getId()), tfCustomFlowEntity.getId(), approveIdList.toString().replaceAll("\\[", "").replaceAll("\\]", ""));

                    nextFlowTaskResult = this.getNextFlowTaskResult(nrOfCompletedInstances,
                            nrOfInstances,
                            currFlowTaskResult,
                            nextFlowTaskResult,
                            currSetup,
                            setupMap,
                            approve);

                    this.approvePass(currFlowTaskResult,
                            setupMap,
                            multiInstance,
                            nrOfCompletedInstances,
                            nrOfInstances,
                            tfFlowApprove,
                            tfFlowLog,
                            nextFlowTaskResult,
                            map,
                            approve);

                    map.put(DefaultInstanceConvertToMultiInstance.DEFAULT_ASSIGNEE_LIST + nextFlowTaskResult.getSetup(), approveIdList);
                    setupMap.put(task.getTaskDefinitionKey(), currFlowTaskResult);
                    flowResult.setSetupMap(setupMap);
                    tfCustomFlowEntity.setFlowProcessVar(JacksonUtil.parseJson(flowResult));
                    if (approve == FlowApprovePassEnum.START) {
                        taskService.complete(task.getId(), map);
                    }
                    tfFlowLogList.add(tfFlowLog);
                }
            }
        }
        tfCustomFlowEntityService.saveOrUpdate(tfCustomFlowEntity);
        tfFlowApproveService.saveOrUpdate(tfFlowApprove);
        tfFlowLogService.saveBatch(tfFlowLogList);
        return Collections.singletonMap("success", true);
    }

    /**
     * 审批通过
     *
     * @param currFlowTaskResult
     * @param setupMap
     * @param multiInstance
     * @param nrOfCompletedInstances
     * @param nrOfInstances
     * @param tfFlowApprove
     * @param tfFlowLog
     * @param nextFlowTaskResult
     * @param map
     * @param approvePass
     */
    public void approvePass(FlowTaskResult currFlowTaskResult,
                            Map<String, FlowTaskResult> setupMap,
                            boolean multiInstance,
                            Integer nrOfCompletedInstances,
                            Integer nrOfInstances,
                            TFFlowApprove tfFlowApprove,
                            TFFlowLog tfFlowLog,
                            FlowTaskResult nextFlowTaskResult,
                            Map<String, Object> map,
                            FlowApprovePassEnum approvePass) {
        if (multiInstance) {
            map.computeIfAbsent(FlowApproveOperationEnum.REJECT.getId() + currFlowTaskResult.getSetup(), v -> 0);
        }
        if (currFlowTaskResult.getSetup().intValue() == setupMap.size() - 1) {
            if (multiInstance && Objects.nonNull(nrOfCompletedInstances)
                    && Objects.nonNull(nrOfInstances)
                    && nrOfCompletedInstances.intValue() == nrOfInstances.intValue()) {
                tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
                tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
            } else if (!multiInstance) {
                tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
                tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
            } else {
                tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
                tfFlowApprove.setSetup(nextFlowTaskResult.getSetup());
            }
        } else {
            if (approvePass == FlowApprovePassEnum.DRAFT || approvePass == FlowApprovePassEnum.START) {
                tfFlowApprove.setStatus(FlowStatusEnum.valueOf(approvePass.name()));
            } else {
                tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
            }
            tfFlowApprove.setSetup(nextFlowTaskResult.getSetup());
        }
        if (tfFlowApprove.getStatus() == FlowStatusEnum.START) {
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.START);
        } else if (tfFlowApprove.getStatus() == FlowStatusEnum.DRAFT) {
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.DRAFT);
        } else {
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
        }
        map.put(SystemConstant.APPROVE, nextFlowTaskResult.getTaskKey());
    }

    /**
     * 获取流程下一节点
     *
     * @param nrOfCompletedInstances
     * @param nrOfInstances
     * @param currFlowTaskResult
     * @param nextFlowTaskResult
     * @param currSetup
     * @param setupMap
     * @param approvePass
     * @return
     */
    public FlowTaskResult getNextFlowTaskResult(Integer nrOfCompletedInstances,
                                                Integer nrOfInstances,
                                                FlowTaskResult currFlowTaskResult,
                                                FlowTaskResult nextFlowTaskResult,
                                                int currSetup,
                                                Map<String, FlowTaskResult> setupMap,
                                                FlowApprovePassEnum approvePass) {
        if (approvePass == FlowApprovePassEnum.PASS || approvePass == FlowApprovePassEnum.START || approvePass == FlowApprovePassEnum.DRAFT) {
            if (Objects.nonNull(nrOfCompletedInstances)
                    && Objects.nonNull(nrOfInstances)
                    && nrOfCompletedInstances.intValue() != nrOfInstances.intValue() || approvePass == FlowApprovePassEnum.DRAFT) {
                nextFlowTaskResult = currFlowTaskResult;
            } else {
                nextFlowTaskResult = this.getNextFlowTaskResult(setupMap, currSetup, nextFlowTaskResult);
            }
        } else if (approvePass == FlowApprovePassEnum.REJECT || approvePass == FlowApprovePassEnum.CANCEL) {
            nextFlowTaskResult = this.getNextFlowTaskResult(setupMap, currSetup, nextFlowTaskResult);
        }
        Optional.ofNullable(nextFlowTaskResult).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("该节点不能审批通过"));
        return nextFlowTaskResult;
    }

    /**
     * 获取下级审核节点
     *
     * @param setupMap
     * @param currSetup
     * @param nextFlowTaskResult
     * @return
     */
    public FlowTaskResult getNextFlowTaskResult(Map<String, FlowTaskResult> setupMap,
                                                int currSetup,
                                                FlowTaskResult nextFlowTaskResult) {
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResultTemp = GsonUtil.fromJson(GsonUtil.toJson(entry.getValue()), FlowTaskResult.class);
            if (flowTaskResultTemp.getSetup().intValue() == currSetup) {
                nextFlowTaskResult = flowTaskResultTemp;
                break;
            }
        }
        return nextFlowTaskResult;
    }

    @ApiOperation(value = "获取待办接口")
    @RequestMapping(value = "/getTaskList", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getTaskList(@ApiParam(value = "用户id", required = true) @RequestParam String
                                                   userId, HttpServletRequest request) {
        Map<String, Object> varMap = new HashMap<>();
        List<Task> taskList = taskService.createTaskQuery().taskAssignee(userId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String flowTaskId = t.getId();
                String processInstanceId = t.getProcessInstanceId();
                String processDefinitionId = t.getProcessDefinitionId();
                log.info("用户id:{},taskId:{},flowId:{},processDefinitionId:{},formKey:{}", userId, flowTaskId, processInstanceId, processDefinitionId, t.getFormKey());
                if (Objects.nonNull(t.getFormKey())) {
                    Object o = formService.getRenderedTaskForm(flowTaskId);
                    if (Objects.nonNull(o)) {
                        log.info("task formProperties:{}", JacksonUtil.parseJson(o));
                        List<FlowFormWidgetResult> list = JSONArray.parseArray(o.toString(), FlowFormWidgetResult.class);
                        formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, userId, null));
                    }
                }
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    @ApiOperation(value = "流程提交接口")
    @RequestMapping(value = "/complete", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> complete(@ApiParam(value = "流程taskId", required = true) @RequestParam String taskId,
                                        @ApiParam(value = "上传文件") @RequestParam(required = false) MultipartFile[] files, HttpServletRequest
                                                request) {
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
        Map<String, Object> varMap = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        varMap.putAll(formDataMap);
        String flowTaskId = task.getId();
        String processInstanceId = task.getProcessInstanceId();
        String processDefinitionId = task.getProcessDefinitionId();
        formService.submitTaskFormData(flowTaskId, formDataMap);
        List<Task> taskList = taskService.createTaskQuery().processInstanceId(processInstanceId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String nextFlowTaskId = t.getId();
                Object o = formService.getRenderedTaskForm(nextFlowTaskId);
                log.info("userId:{},taskId:{},flowId:{},processDefinitionId:{}", t.getAssignee(), nextFlowTaskId, processInstanceId, processDefinitionId);
                if (Objects.nonNull(o)) {
                    log.info("task formProperties:{}", JacksonUtil.parseJson(o));
                    List<FlowFormWidgetResult> list = JSONArray.parseArray(o.toString(), FlowFormWidgetResult.class);
                    formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, t.getAssignee(), null));
                }
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    @ApiOperation(value = "结束流程接口")
    @RequestMapping(value = "/deleteProcessInstance", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public boolean deleteProcessInstance(String runId) {
        return activitiConsumerService.deleteProcessInstance(runId);
    }
}